import os
import re
import yaml
import dotenv
import pathlib
from easydict import EasyDict
from typing import Any


class Konfig:
    def __init__(self, path: str, konfig_file: str):
        self.path = path
        self.konfig_file = konfig_file

    def load_konfig(self):
        dotenv.load_dotenv('.env')
        self.load_file()

    def load_file(self):
        try:
            file_path = pathlib.Path(self.path, self.konfig_file)
            with open(file_path, 'r') as file:
                if file_path.suffix == '.yaml':
                    cfg = yaml.safe_load(file)
            self.set_konfig(cfg)
        except Exception as err:
            raise err

    def set_konfig(self, cfg: dict):
        cfg = load_env_variables(cfg)
        setattr(self, self.konfig_file.split('.')[0], EasyDict(cfg))


def load_env_variables(node: Any) -> Any:
    if isinstance(node, (dict, EasyDict)):
        for k, v in node.items():
            node[k] = load_env_variables(v)
        return node
    elif isinstance(node, list):
        return [load_env_variables(n) for n in node]
    elif isinstance(node, str):
        for env in re.findall(r"\$\w+", node):
            val = os.environ.get(env[1:], None)
            if val is not None:
                node = node.replace(env, val)
        return node
    else:
        return node



from os.path import join, dirname
from setuptools import setup, find_packages
import meinkonfig


def read(name):
    with open(join(dirname(__file__), name), encoding='utf-8') as file:
        return file.read()


setup(
    name='meinkonfig',
    packages=find_packages(),
    version=meinkonfig.__version__,
    author='vodyanoy',
    author_email='vodyanoy420@gmail.com',
    maintainer='vodyanoy',
    maintainer_email='vodyanoy420@gmail.com',
    url='https://github.com/vodyanoy420/meinkonfig',
    license='MIT',
    keywords=['config', 'yaml', 'yamlconfig', 'meinkonfig'],
    platforms=['Linux', 'Unix', 'Mac OS', 'Windows'],
    classifiers=[
        'Programming Language :: Python :: 3.11',
    ],
    description='YAML config reader',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    install_requires=[
        "setuptools~=68.2.0",
        "PyYAML~=6.0.1",
        "python-dotenv~=1.0.0",
        "pathlib~=1.0.1",
        "easydict~=1.11"
    ],
)
